@@ -13,11 +13,11 @@ use crate::{
 		header::{Header, HeaderConfig},
 		reg_entry::RegistryEntry,
 		result::InternalResult,
+		compressor::{Compressor, CompressionAlgorithm},
 	},
 };
 
 use ed25519_dalek as esdalek;
-use lz4_flex as lz4;
 
 /// A wrapper for loading data from archive sources.
 /// It also provides query functions for fetching `Resources` and `RegistryEntry`s.
@@ -54,13 +54,11 @@ impl<T: Seek + Read> Archive<T> {
 	///  - The archive fails to validate
 	///  - `io` errors
 	///  - If any `ID`s are not valid UTF-8
-	pub fn with_config(
-		mut handle: T, config: &HeaderConfig,
-	) -> InternalResult<Archive<T>> {
+	pub fn with_config(mut handle: T, config: &HeaderConfig) -> InternalResult<Archive<T>> {
 		let header = Header::from_handle(&mut handle)?;
 		Header::validate(&header, config)?;
 
-		// Generate and store Registry EntriesF
+		// Generate and store Registry Entries
 		let mut use_decryption = false;
 		let mut entries = HashMap::new();
 
@@ -116,6 +114,8 @@ impl<T: Seek + Read> Archive<T> {
 	pub fn fetch_write<W: Write>(
 		&mut self, id: &str, mut target: W,
 	) -> InternalResult<(Flags, u8, bool)> {
+		/* Literally the hottest function in the block (🕶) */
+
 		if let Some(entry) = self.fetch_entry(id) {
 			let handle = &mut self.handle;
 			let mut is_secure = false;
@@ -155,28 +155,37 @@ impl<T: Seek + Read> Archive<T> {
 					return Err(InternalError::NoKeypairError(format!("Encountered encrypted Leaf: {} but no decryption key(public key) was provided", id)));
 				}
 			}
+
 			// 2: Decompression layer
 			if entry.flags.contains(Flags::COMPRESSED_FLAG) {
-				let mut rdr = lz4::frame::FrameDecoder::new(raw.as_slice());
-				let mut buffer = vec![];
-				rdr.read_to_end(&mut buffer)?;
-
-				raw = buffer;
+				raw = if entry.flags.contains(Flags::LZ4_COMPRESSED) {
+					Compressor::new(raw.as_slice()).decompress(CompressionAlgorithm::LZ4)?
+				} else if entry.flags.contains(Flags::BROTLI_COMPRESSED) {
+					Compressor::new(raw.as_slice()).decompress(CompressionAlgorithm::Brotli(0))?
+				} else if entry.flags.contains(Flags::SNAPPY_COMPRESSED) {
+					Compressor::new(raw.as_slice()).decompress(CompressionAlgorithm::Snappy)?
+				} else {
+					return InternalResult::Err(InternalError::DeCompressionError(
+						"Unspecified compression algorithm bit".to_string(),
+					));
+				};
 			};
+
 			// 3: Deref layer, dereferences link leafs
 			// NOTE: This may break the upcoming cache functionality in `vf`. So `vf` must check for linked `Leaf`s
 			if entry.flags.contains(Flags::LINK_FLAG) {
 				let mut target_id = String::new();
 				raw.as_slice().read_to_string(&mut target_id)?;
 
-				// Prevent cyclic hell
 				match self.fetch_entry(target_id.as_str()) {
+					// Prevents cyclic hell
 					Some(alias) if alias.flags.contains(Flags::LINK_FLAG) => {
 						return Err(InternalError::CyclicLinkReferenceError(
 							id.to_string(),
 							target_id.to_string(),
 						));
 					}
+
 					Some(_) => return self.fetch_write(&target_id, target),
 					None => {
 						return Err(InternalError::MissingResourceError(format!(
@@ -201,8 +210,8 @@ impl<T: Seek + Read> Archive<T> {
 	/// Fetch a `RegistryEntry` from this `Archive`.
 	/// This can be used for debugging, as the `RegistryEntry` holds information about some data within a source.
 	/// ### `None` case:
-	/// If no entry with the given ID exists then None is returned.
-	pub fn fetch_entry(&mut self, id: &str) -> Option<RegistryEntry> {
+	/// If no entry with the given ID exists then `None` is returned.
+	pub fn fetch_entry(&self, id: &str) -> Option<RegistryEntry> {
 		match self.entries.get(id) {
 			Some(entry) => Some(entry.clone()),
 			None => None,
