@@ -1,21 +1,21 @@
-use std::io::{self, BufWriter, Write, Read, Seek, SeekFrom};
+use std::io::{BufWriter, Write, Read, Seek, SeekFrom};
 use std::collections::HashSet;
 
 mod config;
 pub use config::BuilderConfig;
 use super::leaf::{Leaf, CompressMode};
+use crate::global::compressor::Compressor;
 use crate::global::error::InternalError;
 use crate::global::result::InternalResult;
 use crate::{
 	global::{edcryptor::EDCryptor, header::Header, reg_entry::RegistryEntry, flags::Flags},
 };
 
-use lz4_flex as lz4;
 use ed25519_dalek::Signer;
 
 /// The archive builder. Provides an interface with which one can configure and build out valid `vach` archives.
 pub struct Builder<'a> {
-	leafs: Vec<Leaf<'a>>,
+	pub(crate) leafs: Vec<Leaf<'a>>,
 	pub(crate) id_set: HashSet<String>,
 	leaf_template: Leaf<'a>,
 }
@@ -49,20 +49,25 @@ impl<'a> Builder<'a> {
 		Ok(())
 	}
 
+	/// Removes all the `Leaf`s from the `Builder`. Leaves the `template` intact. Use this to re-use `Builder`s instead of instantiating new ones
+	pub fn clear(&mut self) {
+		self.id_set.clear();
+		self.leafs.clear();
+	}
+
 	/// Loads all files from a directory, parses them into `Leaf`s and appends them into the processing queue.
 	/// An optional `Leaf` is passed as a template from which the new `Leaf`s shall implement, pass `None` to use the `Builder` internal default template.
 	/// Appended `Leaf`s have an `ID` in the form of of: `directory_name/file_name`. For example: "sounds/footstep.wav", "sample/script.data"
 	/// ## Errors
 	/// - Any of the underlying calls to the filesystem fail.
 	/// - The internal call to `Builder::add_leaf()` returns an error.
-	pub fn add_dir(&mut self, path: &str, template: Option<&Leaf>) -> InternalResult<()> {
+	pub fn add_dir(&mut self, path: &str, template: Option<&Leaf<'a>>) -> InternalResult<()> {
 		use std::fs;
 
 		let directory = fs::read_dir(path)?;
 		for file in directory {
 			let uri = file?.path();
 
-			// BUG: Fix this DUMB DUMB code
 			let v = uri
 				.iter()
 				.map(|u| String::from(u.to_str().unwrap()))
@@ -129,21 +134,17 @@ impl<'a> Builder<'a> {
 		let mut reg_buffer = Vec::new();
 
 		// Calculate the size of the registry and check for `Leaf`s that request for encryption
-		let mut leaf_offset = self
-			.leafs
-			.iter()
-			.map(|leaf| {
-				// The size of it's ID, the minimum size of an entry without a signature, and the size of a signature only if a signature is incorporated into the entry
-				leaf.id.len()
-					+ RegistryEntry::MIN_SIZE
-					+ (if config.keypair.is_some() {
-						crate::SIGNATURE_LENGTH
-					} else {
-						0
-					})
-			})
-			.reduce(|l1, l2| l1 + l2)
-			.unwrap_or(0) + Header::BASE_SIZE;
+		let mut leaf_offset =
+			self.leafs
+				.iter()
+				.map(|leaf| {
+					// The size of it's ID, the minimum size of an entry without a signature, and the size of a signature only if a signature is incorporated into the entry
+					leaf.id.len()
+						+ RegistryEntry::MIN_SIZE
+						+ (if config.keypair.is_some() { crate::SIGNATURE_LENGTH } else { 0 })
+				})
+				.reduce(|l1, l2| l1 + l2)
+				.unwrap_or(0) + Header::BASE_SIZE;
 
 		// Start at the very start of the file
 		target.seek(SeekFrom::Start(0))?;
@@ -212,23 +213,23 @@ impl<'a> Builder<'a> {
 					leaf.handle.read_to_end(&mut leaf_bytes)?;
 				}
 				CompressMode::Always => {
-					let mut compressor = lz4::frame::FrameEncoder::new(leaf_bytes);
-					io::copy(&mut leaf.handle, &mut compressor)?;
-					leaf_bytes = compressor.finish()?;
+					leaf_bytes = Compressor::new(&mut leaf.handle).compress(leaf.compression_algo)?;
+
 					entry.flags.force_set(Flags::COMPRESSED_FLAG, true);
+					entry.flags.force_set(leaf.compression_algo.into(), true);
 				}
 				CompressMode::Detect => {
 					let mut buffer = Vec::new();
 					leaf.handle.read_to_end(&mut buffer)?;
 
-					let mut compressor = lz4::frame::FrameEncoder::new(Vec::new());
-					io::copy(&mut buffer.as_slice(), &mut compressor)?;
-					let mut compressed_data = compressor.finish()?;
-
+					let compressed_data = Compressor::new(buffer.as_slice()).compress(leaf.compression_algo)?;
 					let ratio = compressed_data.len() as f32 / buffer.len() as f32;
+
 					if ratio < 1f32 {
 						entry.flags.force_set(Flags::COMPRESSED_FLAG, true);
-						leaf_bytes.append(&mut compressed_data);
+						entry.flags.force_set(leaf.compression_algo.into(), true);
+
+						leaf_bytes = compressed_data;
 					} else {
 						buffer.as_slice().read_to_end(&mut leaf_bytes)?;
 					};
@@ -289,6 +290,11 @@ impl<'a> Builder<'a> {
 			// Write to the registry
 			reg_buffer.write_all(&entry_bytes)?;
 
+			// Call the progress callback bound within the `BuilderConfig`
+			if let Some(callback) = &config.progress_callback {
+				callback(&leaf.id, glob_length, &entry);
+			}
+
 			// Update offsets
 			size += entry_bytes.len();
 		}
